@@ -21,7 +21,7 @@ sonarr_logger = get_logger("sonarr")
 # Use a session for better performance
 session = requests.Session()
 
-def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, method: str = "GET", data: Dict = None) -> Any:
+def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, method: str = "GET", data: Dict = None, count_api: bool = True) -> Any:
     """
     Make a request to the Sonarr API.
     
@@ -49,8 +49,6 @@ def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, met
         # Construct the full URL properly
         full_url = f"{api_url.rstrip('/')}/api/v3/{endpoint.lstrip('/')}"
         
-        sonarr_logger.debug(f"Making {method} request to: {full_url}")
-        
         # Set up headers with User-Agent to identify Huntarr
         headers = {
             "X-Api-Key": api_key,
@@ -58,9 +56,6 @@ def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, met
             "User-Agent": "Huntarr/1.0 (https://github.com/plexguide/Huntarr.io)"
         }
         
-        # Log the User-Agent for debugging
-        sonarr_logger.debug(f"Using User-Agent: {headers['User-Agent']}")
-        
         # Get SSL verification setting
         verify_ssl = get_ssl_verify_setting()
         
@@ -83,6 +78,14 @@ def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, met
             # Check for successful response
             response.raise_for_status()
             
+            # Increment API counter only if count_api is True and request was successful
+            if count_api:
+                try:
+                    from src.primary.stats_manager import increment_hourly_cap
+                    increment_hourly_cap("sonarr")
+                except Exception as e:
+                    sonarr_logger.warning(f"Failed to increment API counter for sonarr: {e}")
+            
             # Check if there's any content before trying to parse JSON
             if response.content:
                 try:
@@ -154,7 +157,7 @@ def get_system_status(api_url: str, api_key: str, api_timeout: int) -> Dict:
     Returns:
         System status information or empty dict if request failed
     """
-    response = arr_request(api_url, api_key, api_timeout, "system/status")
+    response = arr_request(api_url, api_key, api_timeout, "system/status", count_api=False)
     if response:
         return response
     return {}
@@ -177,7 +180,7 @@ def get_series(api_url: str, api_key: str, api_timeout: int, series_id: Optional
     else:
         endpoint = "series"
     
-    return arr_request(api_url, api_key, api_timeout, endpoint)
+    return arr_request(api_url, api_key, api_timeout, endpoint, count_api=False)
 
 def get_episode(api_url: str, api_key: str, api_timeout: int, episode_id: int) -> Dict:
     """
@@ -192,7 +195,7 @@ def get_episode(api_url: str, api_key: str, api_timeout: int, episode_id: int) -
     Returns:
         Episode information or empty dict if request failed
     """
-    response = arr_request(api_url, api_key, api_timeout, f"episode/{episode_id}")
+    response = arr_request(api_url, api_key, api_timeout, f"episode/{episode_id}", count_api=False)
     if response:
         return response
     return {}
@@ -209,7 +212,7 @@ def get_queue(api_url: str, api_key: str, api_timeout: int) -> List:
     Returns:
         Queue information or empty list if request failed
     """
-    response = arr_request(api_url, api_key, api_timeout, "queue")
+    response = arr_request(api_url, api_key, api_timeout, "queue", count_api=False)
     if not response or "records" not in response:
         return []
     
@@ -241,7 +244,7 @@ def get_calendar(api_url: str, api_key: str, api_timeout: int, start_date: Optio
     if params:
         endpoint = f"{endpoint}?{'&'.join(params)}"
     
-    response = arr_request(api_url, api_key, api_timeout, endpoint)
+    response = arr_request(api_url, api_key, api_timeout, endpoint, count_api=False)
     if response:
         return response
     return []
@@ -259,7 +262,7 @@ def command_status(api_url: str, api_key: str, api_timeout: int, command_id: Uni
     Returns:
         Command status information or empty dict if request failed
     """
-    response = arr_request(api_url, api_key, api_timeout, f"command/{command_id}")
+    response = arr_request(api_url, api_key, api_timeout, f"command/{command_id}", count_api=False)
     if response:
         return response
     return {}
@@ -282,7 +285,8 @@ def get_missing_episodes(api_url: str, api_key: str, api_timeout: int, monitored
             params = {
                 "page": page,
                 "pageSize": page_size,
-                "includeSeries": "true"
+                "includeSeries": "true",
+                "monitored": monitored_only
             }
             
             # Add series ID filter if provided
@@ -405,7 +409,8 @@ def get_cutoff_unmet_episodes(api_url: str, api_key: str, api_timeout: int, moni
                 "pageSize": page_size,
                 "includeSeries": "true", # Include series info for filtering
                 "sortKey": "airDateUtc",
-                "sortDir": "asc"
+                "sortDir": "asc",
+                "monitored": monitored_only
             }
             url = f"{api_url}/api/v3/{endpoint}"
             sonarr_logger.debug(f"Requesting cutoff unmet page {page} (attempt {retry_count+1}/{retries_per_page+1})")
@@ -545,7 +550,8 @@ def get_cutoff_unmet_episodes_random_page(api_url: str, api_key: str, api_timeou
     params = {
         "page": 1,
         "pageSize": 1,
-        "includeSeries": "true"  # Include series info for filtering
+        "includeSeries": "true",  # Include series info for filtering
+        "monitored": monitored_only
     }
     url = f"{api_url}/api/v3/{endpoint}"
     
@@ -576,7 +582,8 @@ def get_cutoff_unmet_episodes_random_page(api_url: str, api_key: str, api_timeou
         params = {
             "page": random_page,
             "pageSize": page_size,
-            "includeSeries": "true"
+            "includeSeries": "true",
+            "monitored": monitored_only
         }
         
         response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
@@ -640,7 +647,8 @@ def get_missing_episodes_random_page(api_url: str, api_key: str, api_timeout: in
     params = {
         "page": 1,
         "pageSize": 1,
-        "includeSeries": "true"  # Include series info for filtering
+        "includeSeries": "true",  # Include series info for filtering
+        "monitored": monitored_only
     }
     url = f"{api_url}/api/v3/{endpoint}"
     
@@ -682,7 +690,8 @@ def get_missing_episodes_random_page(api_url: str, api_key: str, api_timeout: in
                 params = {
                     "page": random_page,
                     "pageSize": page_size,
-                    "includeSeries": "true"
+                    "includeSeries": "true",
+                    "monitored": monitored_only
                 }
                 
                 if series_id is not None:
@@ -756,6 +765,17 @@ def search_episode(api_url: str, api_key: str, api_timeout: int, episode_ids: Li
     if not episode_ids:
         sonarr_logger.warning("No episode IDs provided for search.")
         return None
+    
+    # Check API limit before making request
+    try:
+        from src.primary.stats_manager import check_hourly_cap_exceeded
+        if check_hourly_cap_exceeded("sonarr"):
+            sonarr_logger.warning(f"🛑 Sonarr API hourly limit reached - skipping episode search for {len(episode_ids)} episodes")
+            return None
+    except Exception as e:
+        sonarr_logger.error(f"Error checking hourly API cap: {e}")
+        # Continue with request if cap check fails - safer than skipping
+    
     try:
         endpoint = f"{api_url}/api/v3/command"
         payload = {
@@ -766,6 +786,15 @@ def search_episode(api_url: str, api_key: str, api_timeout: int, episode_ids: Li
         response.raise_for_status()
         command_id = response.json().get('id')
         sonarr_logger.info(f"Triggered Sonarr search for episode IDs: {episode_ids}. Command ID: {command_id}")
+        
+        # Increment API counter after successful request
+        try:
+            from src.primary.stats_manager import increment_hourly_cap
+            increment_hourly_cap("sonarr", 1)
+            sonarr_logger.debug(f"Incremented Sonarr hourly API cap for episode search ({len(episode_ids)} episodes)")
+        except Exception as cap_error:
+            sonarr_logger.error(f"Failed to increment hourly API cap for episode search: {cap_error}")
+        
         return command_id
     except requests.exceptions.RequestException as e:
         sonarr_logger.error(f"Error triggering Sonarr search for episode IDs {episode_ids}: {e}")
@@ -864,6 +893,17 @@ def get_series_by_id(api_url: str, api_key: str, api_timeout: int, series_id: in
 
 def search_season(api_url: str, api_key: str, api_timeout: int, series_id: int, season_number: int) -> Optional[Union[int, str]]:
     """Trigger a search for a specific season in Sonarr."""
+    
+    # Check API limit before making request
+    try:
+        from src.primary.stats_manager import check_hourly_cap_exceeded
+        if check_hourly_cap_exceeded("sonarr"):
+            sonarr_logger.warning(f"🛑 Sonarr API hourly limit reached - skipping season search for series {series_id}, season {season_number}")
+            return None
+    except Exception as e:
+        sonarr_logger.error(f"Error checking hourly API cap: {e}")
+        # Continue with request if cap check fails - safer than skipping
+    
     try:
         endpoint = f"{api_url}/api/v3/command"
         payload = {
@@ -930,7 +970,8 @@ def get_cutoff_unmet_episodes_for_series(api_url: str, api_key: str, api_timeout
                 "includeSeries": "true", # Include series info for filtering
                 "sortKey": "airDateUtc",
                 "sortDir": "asc",
-                "seriesId": series_id  # Filter by series ID - this limits results to only this series
+                "seriesId": series_id,  # Filter by series ID - this limits results to only this series
+                "monitored": monitored_only
             }
             url = f"{api_url}/api/v3/{endpoint}"
             sonarr_logger.debug(f"Requesting cutoff unmet page {page} for series {series_id} (attempt {retry_count+1}/{retries_per_page+1})")
@@ -1135,4 +1176,155 @@ def get_series_with_missing_episodes(api_url: str, api_key: str, api_timeout: in
     
     selection_mode = "RANDOM" if random_mode else "SEQUENTIAL"        
     sonarr_logger.info(f"Examined {examined_count} series ({selection_mode} mode) and found {len(series_with_missing)} with missing episodes")
-    return series_with_missing
\ No newline at end of file
+    return series_with_missing
+
+def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
+    """
+    Get existing tag ID or create a new tag in Sonarr.
+    
+    Args:
+        api_url: The base URL of the Sonarr API
+        api_key: The API key for authentication
+        api_timeout: Timeout for the API request
+        tag_label: The label/name of the tag to create or find
+        
+    Returns:
+        The tag ID if successful, None otherwise
+    """
+    try:
+        # First, check if the tag already exists
+        response = arr_request(api_url, api_key, api_timeout, "tag", count_api=False)
+        if response:
+            for tag in response:
+                if tag.get('label') == tag_label:
+                    tag_id = tag.get('id')
+                    sonarr_logger.debug(f"Found existing tag '{tag_label}' with ID: {tag_id}")
+                    return tag_id
+        
+        # Tag doesn't exist, create it
+        tag_data = {"label": tag_label}
+        response = arr_request(api_url, api_key, api_timeout, "tag", method="POST", data=tag_data, count_api=False)
+        if response and 'id' in response:
+            tag_id = response['id']
+            sonarr_logger.info(f"Created new tag '{tag_label}' with ID: {tag_id}")
+            return tag_id
+        else:
+            sonarr_logger.error(f"Failed to create tag '{tag_label}'. Response: {response}")
+            return None
+            
+    except Exception as e:
+        sonarr_logger.error(f"Error managing tag '{tag_label}': {e}")
+        return None
+
+def add_tag_to_series(api_url: str, api_key: str, api_timeout: int, series_id: int, tag_id: int) -> bool:
+    """
+    Add a tag to a series in Sonarr.
+    
+    Args:
+        api_url: The base URL of the Sonarr API
+        api_key: The API key for authentication
+        api_timeout: Timeout for the API request
+        series_id: The ID of the series to tag
+        tag_id: The ID of the tag to add
+        
+    Returns:
+        True if successful, False otherwise
+    """
+    try:
+        # First get the current series data
+        series_data = arr_request(api_url, api_key, api_timeout, f"series/{series_id}", count_api=False)
+        if not series_data:
+            sonarr_logger.error(f"Failed to get series data for ID: {series_id}")
+            return False
+        
+        # Check if the tag is already present
+        current_tags = series_data.get('tags', [])
+        if tag_id in current_tags:
+            sonarr_logger.debug(f"Tag {tag_id} already exists on series {series_id}")
+            return True
+        
+        # Add the new tag to the list
+        current_tags.append(tag_id)
+        series_data['tags'] = current_tags
+        
+        # Update the series with the new tags
+        response = arr_request(api_url, api_key, api_timeout, f"series/{series_id}", method="PUT", data=series_data, count_api=False)
+        if response:
+            sonarr_logger.debug(f"Successfully added tag {tag_id} to series {series_id}")
+            return True
+        else:
+            sonarr_logger.error(f"Failed to update series {series_id} with tag {tag_id}")
+            return False
+            
+    except Exception as e:
+        sonarr_logger.error(f"Error adding tag {tag_id} to series {series_id}: {e}")
+        return False
+
+def tag_processed_series(api_url: str, api_key: str, api_timeout: int, series_id: int, tag_label: str = "huntarr-missing") -> bool:
+    """
+    Tag a series in Sonarr with the specified tag.
+    
+    Args:
+        api_url: The base URL of the Sonarr API
+        api_key: The API key for authentication
+        api_timeout: Timeout for the API request
+        series_id: The ID of the series to tag
+        tag_label: The tag to apply (huntarr-missing, huntarr-upgraded, huntarr-shows-missing)
+        
+    Returns:
+        True if the tagging was successful, False otherwise
+    """
+    try:
+        # Get or create the tag
+        tag_id = get_or_create_tag(api_url, api_key, api_timeout, tag_label)
+        if tag_id is None:
+            sonarr_logger.error(f"Failed to get or create tag '{tag_label}' in Sonarr")
+            return False
+            
+        # Add the tag to the series
+        success = add_tag_to_series(api_url, api_key, api_timeout, series_id, tag_id)
+        if success:
+            sonarr_logger.debug(f"Successfully tagged Sonarr series {series_id} with '{tag_label}'")
+            return True
+        else:
+            sonarr_logger.error(f"Failed to add tag '{tag_label}' to Sonarr series {series_id}")
+            return False
+            
+    except Exception as e:
+        sonarr_logger.error(f"Error tagging Sonarr series {series_id} with '{tag_label}': {e}")
+        return False
+
+def get_quality_profiles(api_url: str, api_key: str, api_timeout: int) -> Optional[List[Dict]]:
+    """
+    Get all quality profiles configured in Sonarr.
+
+    Args:
+        api_url: The base URL of the Sonarr API
+        api_key: The API key for authentication
+        api_timeout: Timeout for the API request
+
+    Returns:
+        A list of quality profile objects, or None if the request failed.
+        Each profile contains: id, name, upgradeAllowed, cutoff, items, etc.
+    """
+    try:
+        sonarr_logger.debug("Fetching quality profiles from Sonarr...")
+        
+        # Use the qualityProfile endpoint - this doesn't count toward API limits since it's configuration data
+        profiles = arr_request(api_url, api_key, api_timeout, "qualityProfile", count_api=False)
+        
+        if profiles is None:
+            sonarr_logger.error("Failed to retrieve quality profiles from Sonarr API.")
+            return None
+        
+        sonarr_logger.debug(f"Found {len(profiles)} quality profiles in Sonarr")
+        
+        # Log profile names for debugging
+        profile_names = [profile.get('name', 'Unknown') for profile in profiles]
+        sonarr_logger.debug(f"Quality profiles: {', '.join(profile_names)}")
+        
+        return profiles
+        
+    except Exception as e:
+        sonarr_logger.error(f"Error retrieving quality profiles: {str(e)}")
+        return None
\ No newline at end of file
